#ifndef __EXT2_H
#define __EXT2_H

#include <fs_ext2.h>
#include <fs.h>
#include <block_cache.h>

/*
 * Estan son las features obligatorias que nosotros soportamos, 
 * si hay alguna otra que no soportamos abortamos
 */
#define REQUIRED_FLAGS_SUPPORT EXT2_FEATURE_DIR_TYPE 

// Tamaño inicial de la cache
#define EXT2_INITIAL_CACHE_COUNT 16

// Tamaño maximo del nombre de un archivo
#define EXT2_MAX_FILENAME_SIZE   256

typedef struct fs_file {
    ext2_inode *inode;         // Puntero al inodo de este archivo
    uint64_t remaining;        // Cantidad de bytes que quedan por leer
    uint64_t block_offset;     // Offset actual dentro del bloque
    uint32_t block_number;     // Número de bloque actual
    uint32_t block_index;      // Indice de puntero a bloque actual
    uint32_t single_index;     // Indice del bloque indirecto simple actual
    uint32_t double_index;     // Indice del bloque indirecto doble actual
    uint32_t *singly_indirect; // Puntero al bloque indirecto simple
    uint32_t *doubly_indirect; // Puntero al bloque indirecto doble
    uint32_t *triply_indirect; // Puntero al bloque indirecto triple
    /* Usado en ext2_write */
    uint32_t inode_number;     // Numero de inodo del archivo
    uint64_t prealloc_count;   // Cantidad de bloques prealloc
    uint32_t prealloc_block;   // Primer bloque prealloc
} fs_file;

typedef struct fs_dir {
    fs_file file;              // Struct del directory file
    dir_entry data;            // Datos de la entrada en la que estamos
    /* Usado en ext2_write */
    uint16_t rec_len;          // Longitud de la entrada actual
    uint32_t rec_inode;        // Inodo de la entrada actual
} fs_dir;

typedef struct fs_info {
    block_cache* head_cache;   // Cache con el superbloque y la tabla
    block_cache* cache;        // Cache general para cargar bloques
    ext2_superblock* sb;       // Puntero directo al superbloque
    ext2_group_desc* gt;       // Puntero directo a la tabla de grupos
    uint64_t block_size;       // Tamaño de un bloque pero en bytes
    uint64_t block_shift;      // Shift para dividir por el tamaño de un bloque
    uint64_t group_count;      // Cantidad de grupos de bloque
} fs_info;

// Retorna el tamaño de un archivo dado su inodo
uint64_t ext2_get_size(ext2_inode *inode);
// Retorna el numero de bloque donde se encuentra el superbloque
uint32_t    ext2_parse_superblock(fs_info* info);
// Chequea que los parametros del superbloque esten en orden
void        ext2_check_superblock(ext2_superblock* sb);

ext2_inode* ext2_load_inode(fs_info* info, uint32_t inode_number);
fs_file*    ext2_load_file(ext2_inode *inode, fs_file *file, 
                           uint32_t inode_num);
fs_dir*     ext2_load_dir(ext2_inode *inode, fs_dir *dir, uint32_t inode_num);
void        ext2_unload_file(fs_info* info, fs_file *file);

uint64_t    ext2_read(block_cache* cache, uint64_t block_shift, 
                      fs_file* file, void* buffer, uint64_t count);
int64_t     ext2_seek(block_cache* cache, uint64_t block_shift, 
                      fs_file* file, int64_t offset); 


#endif
