#ifndef __HDD_H
#define __HDD_H

#include <types.h>

// Constantes utiles
#define ATA_SECTOR_SIZE         512L
#define ATA_SECTOR_WORDS        256L

#define ATA_PRIMARY_BASE          0x1F0
#define ATA_PRIMARY_DEVCONTROL    0x3F6
#define ATA_SECONDARY_BASE        0x170
#define ATA_SECONDARY_DEVCONTROL  0x376

#define ATA_PRIMARY      0x00
#define ATA_SECONDARY    0x01

#define IDE_ATA        0x00
#define IDE_ATAPI      0x01
#define IDE_SATA       0x02
#define IDE_SATAPI     0x03
 
#define ATA_MASTER     0x00
#define ATA_SLAVE      0x01


#define ATA_DATA        0x0
#define ATA_ERR         0x1
#define ATA_SECTORCOUNT 0x2
#define ATA_LBALOW      0x3
#define ATA_LBAMID      0x4
#define ATA_LBAHIGH     0x5
#define ATA_DRIVE       0x6
#define ATA_COMSTAT     0x7
#define ATA_STATUS      0x7
#define ATA_SECCOUNT1   0x8
#define ATA_LBA3        0x9
#define ATA_LBA4        0xA
#define ATA_LBA5        0xB
#define ATA_CONTROL     0xC
#define ATA_ALTSTATUS   0xC
#define ATA_DEVADDRESS  0xD


#define ATA_PRIMARY_DATA        (0x0+ATA_PRIMARY_BASE)
#define ATA_PRIMARY_ERR         (0x1+ATA_PRIMARY_BASE)
#define ATA_PRIMARY_SECTORCOUNT (0x2+ATA_PRIMARY_BASE)
#define ATA_PRIMARY_LBALOW      (0x3+ATA_PRIMARY_BASE)
#define ATA_PRIMARY_LBAMID      (0x4+ATA_PRIMARY_BASE)
#define ATA_PRIMARY_LBAHIGH     (0x5+ATA_PRIMARY_BASE)
#define ATA_PRIMARY_DRIVE       (0x6+ATA_PRIMARY_BASE)
#define ATA_PRIMARY_COMSTAT     (0x7+ATA_PRIMARY_BASE)

#define ATA_STATUS_ERR          (1 << 0)
#define ATA_STATUS_DRQ          (1 << 3)
#define ATA_STATUS_SRV          (1 << 4)
#define ATA_STATUS_DF           (1 << 5)
#define ATA_STATUS_RDY          (1 << 6)
#define ATA_STATUS_BSY          (1 << 7)

#define ATA_CTRL_NIEN           (1 << 1)
#define ATA_CTRL_SRST           (1 << 2)
#define ATA_CTRL_HOB            (1 << 7)

#define ATA_COMM_READ_LBA28     0x20
#define ATA_COMM_WRITE_LBA28    0x30
#define ATA_COMM_FLUSHCACHE     0xE7

// Struct identify con info de un disco, solo definimos los campos que interesan
// Tiene que medir exactamente 512 bytes (256 words)
typedef struct {
    uint16_t device_type;
    uint16_t unused0[26];
    char model[40];
    uint16_t unused1[2];
    uint16_t capabilities;
    uint16_t unused2[3];
    uint16_t field_valid;
    uint16_t unused3[6];
    uint32_t max_lba;
    uint16_t unused4[20];
    uint16_t command_sets;
    uint16_t unused5[17];
    uint64_t max_lba_ext;
    uint16_t unused6[152];
} identify_data;

#define ATA_IDENT_DEVICETYPE   0
#define ATA_IDENT_MODEL        27
#define ATA_IDENT_CAPABILITIES 48
#define ATA_IDENT_FIELDVALID   53
#define ATA_IDENT_MAX_LBA      60
#define ATA_IDENT_COMMANDSETS  82
#define ATA_IDENT_MAX_LBA_EXT  100

// Inicializar y detectar discos
uint8_t hdd_init(uint16_t port, uint8_t master, uint8_t dev);
// Leer (usando LBA 28) una cantidad de bytes de un disco a un buffer
void hdd_read(uint32_t lba_address, uint32_t sectors, void* buffer);
// Escribir (usando LBA 28) una cantidad de bytes de un disco a un buffer
void hdd_write(uint32_t lba_address, uint32_t sectors, const void* buffer);

void ide_init(uint32_t bar0, uint32_t bar1, uint32_t bar2, uint32_t bar3,
              uint32_t bar4);
#endif
