#ifndef __IO_H
#define __IO_H

#include <fs.h>
// Cambiar filesystem a gusto
#include <ext2.h>

// Standard file descriptors
#define STDIN   0
#define STDOUT  1
#define STDERR  2

// Seek
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2

// Open modes
#define O_RDONLY 0x0
#define O_WRONLY 0x1
#define O_RDWR   0x2

#define MAX_FILE_DESCRIPTORS 256
#define MAX_DIR_DESCRIPTORS 256

typedef struct fd_entry { 
    fs_stat stat;
    fs_file file;
} fd_entry;

typedef struct dd_entry {
    uint8_t valid;
    fs_dir dir;
} dd_entry;

int64_t     io_init();
int64_t     io_kill();

/* SYSCALLS */
int64_t     read(int64_t fd, void *buf, uint64_t count);
int64_t     write(int64_t fd, const void *buf, uint64_t count);
int64_t     seek(int64_t fd, int64_t offset, int64_t origin);
int64_t     tell(int64_t fd);
uint8_t     type(const char *path);
int64_t     create(const char *path, uint8_t filetype);
int64_t     remove(const char *path); 
int64_t     truncate(int64_t fd, uint64_t length);
int64_t     open(const char *path);
int64_t     close(int64_t fd);
int64_t     opendir(const char *path);
int64_t     closedir(int64_t dd);
int64_t     rewinddir(int64_t dd);
/* Devuelve un puntero al siguiente filename. 
 * NULL si llegó al final de la lista. */
dir_entry * readdir(int64_t dd);

// TODO: Ver si es necesario, devuelve si se llego al final o no del archivo
uint64_t    eof(int64_t fd);
// Setea el current working directory 
int64_t     set_cwd_dir(const char *path);

#endif
