#ifndef __KMALLOC_H__
#define __KMALLOC_H__

#include <types.h>

/* Este struct lo van a usar los bloques del heap, sean libres o no
 * Los que son libres usan ambos campos, los que no usan solo el primer puntero
 * Por lo tanto cuando nos referimos a un bloque ocupado solo vamos a usar next
 * Es como una skip list circular con solo 2 niveles, por ejemplo:
 *
 *     free_ptr
 *         |
 *         V
 *->| FREE  next_free |------------------------------------------->
 *->| BLOCK      next |->| USED BLOCK next |->| USED BLOCK next |->
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *->| FREE  next-free |----------------------> (Al primer bloque libre)
 *->| BLOCK      next |->| DUMMY BLOCK next|-> (Al primer bloque      ) 
 *                               |
 *                               V
 *              Dummy block marca el final de la memoria
 */

typedef struct kheap_header {
    // Puntero al proximo bloque (libre o no)
    struct kheap_header* next;
    // Puntero al proximo bloque libre
    struct kheap_header* next_free;
} __attribute__((packed)) kheap_header;


typedef struct kheap_list_header {
    struct kheap_header* free_ptr;
    kheap_header* start;
    kheap_header* end;
} kheap_list_header;

// Inicializa el heap del kernel
void kheap_init(void);
// Consigue memoria del heap del kernel
void *kmalloc(uint64_t size);
// Devuelve memoria al heap del kernel
void kfree(void * mem);
// Devuelve la cantidad de memoria disponible
uint64_t kheap_available(kheap_list_header* klh);


#endif  /* __KMALLOC_H__ */
