#ifndef __SCREEN_H
#define __SCREEN_H

#include <types.h>

#define TAB_WIDTH 4

#define COLOR_BLACK         0x0
#define COLOR_BLUE          0x1
#define COLOR_GREEN         0x2
#define COLOR_CYAN          0x3
#define COLOR_RED           0x4
#define COLOR_MAGENTA       0x5
#define COLOR_BROWN         0x6
#define COLOR_LIGHTGREY     0x7
#define COLOR_DARKGREY      0x8
#define COLOR_LIGHTBLUE     0x9
#define COLOR_LIGHTGREEN    0xA
#define COLOR_LIGHTCYAN     0xB
#define COLOR_LIGHTRED      0xC
#define COLOR_LIGHTMAGENTA  0xD
#define COLOR_LIGHTBROWN    0xE
#define COLOR_WHITE         0xF

#define SCREEN_FORMAT(f,b)    ((((b) << 4) | (f)) << 8)
#define SCREEN_FORMAT_DEFAULT SCREEN_FORMAT(COLOR_WHITE, COLOR_BLACK)
#define SCREEN_FORMAT_ERROR   SCREEN_FORMAT(COLOR_RED, COLOR_BLACK)

// Limpia la pantalla
void screen_clear(void);
// Coloca el color de fondo y la fuente.
void screen_setmode(uint8_t font, uint8_t back);
// Devuelve la fila del cursor
char screen_getrow(void);
// Devuelve la columna del cursor
char screen_getcol(void);
// Coloca el cursor en un lugar.
// PRE: Las coordenadas pasadas son validas dentro de la memoria de video
void screen_setcursor(uint8_t row, uint8_t col);
// Imprime un caracter a memoria de video. No utiliza format.
void screen_putchar(char c);
void screen_move_back(void);
// Shiftea la pantalla una fila hacia arriba
void screen_shift_up(void);

// Lee exactamente length caracteres de la pantalla o hasta un salto de linea
// Los escribe en buf, devuelve la cantidad de caracteres leidos
uint64_t screen_read(void* buf, uint64_t count);
uint64_t screen_write(const void* buf, uint64_t count);
uint64_t screen_write_err(const void* buf, uint64_t count);

#endif
