#include <core_id.h>
#include <idt.h>
#include <ioapic.h>
#include <ports.h>

static uint32_t ioapic_read_register(uint32_t ioapic, uint32_t index) {
    // En el inicio de la ioapic esta el selector de registros
    VOLATILE_DWORD_ARRAY(ioapic)[0] = index;
    // 4 DWORDS mas adelante esta el registro en si mapeado
    return VOLATILE_DWORD_ARRAY(ioapic)[4];
}

static void ioapic_write_register(uint32_t ioapic, uint32_t index,
                                  uint32_t value) {
    // En el inicio de la ioapic esta el selector de registros
    VOLATILE_DWORD_ARRAY(ioapic)[0] = index;
    // 4 DWORDS mas adelante esta el registro en si mapeado
    VOLATILE_DWORD_ARRAY(ioapic)[4] = value;
}

static void ioapic_write_entry(uint32_t ioapic, uint32_t entry,
                               uint64_t value) {
    // Como las entries son de QWORDS, duplicamos el indice
    uint32_t index = IOAPIC_REDIRECTION_TABLE_INDEX + entry * 2;
    ioapic_write_register(ioapic, index, (uint32_t)(value & 0xFFFFFFFF));
    ioapic_write_register(ioapic, index + 1, (uint32_t)(value >> 32));
}

void initialize_ioapic(uint32_t ioapic, uint8_t bsp_id) {
    // Seleccionamos ICMR
    outb(0x22, 0x70);
    // Seteamos el primer bit para SMP
    outb(0x23, 0x01);
    // Obtenemos el registro de version
    uint32_t version = ioapic_read_register(ioapic, IOAPIC_VERSION_INDEX);
    // Maximum redirection entry
    uint32_t mre = (version >> 16) & 0xFF;
    uint64_t bsp_field = ((uint64_t)bsp_id) << 56;
    uint64_t redirection = bsp_field | IOAPIC_RD_MASK_BIT;
    for (uint32_t i = 0; i <= mre; i++)
        ioapic_write_entry(ioapic, i, redirection);
    // Activamos el teclado IRQ 1
    ioapic_write_entry(ioapic, 1, bsp_field | INT_NUM_KEYBOARD);
    // Activamos el Real Time Clock IRQ 8
    ioapic_write_entry(ioapic, 8, bsp_field | INT_NUM_RTC);
    // Activamos el Primary ATA Hard Disk IRQ 14
    ioapic_write_entry(ioapic, 14, bsp_field | INT_NUM_ATA_PRIMARY);
    // Activamos el Secondary ATA Hard Disk IRQ 15
    ioapic_write_entry(ioapic, 15, bsp_field | INT_NUM_ATA_SECONDARY);
}
