%include "long_mode_macros.mac"
%include "apic_macros.mac"
%include "defines.mac"

BITS 64

;; Keyboard 
extern handle_key_event

;; Timer
extern timer_tick

;; Exception
extern kernel_panic

;; Loader
extern execute_end, reset_stack_awe

;; CMOS
extern cmos_readb

;; LAPIC
extern local_apic_address

;; Syscall
extern exit

;; App Data
extern app_data

;; Mensajes
isr_msg0: db '#DE Divide Error', 0
isr_msg1: db '#DB RESERVED', 0
isr_msg2: db 'NMI Interrupt', 0
isr_msg3: db '#BP Breakpoint', 0
isr_msg4: db '#OF Overflow', 0
isr_msg5: db '#BR BOUND Range Exceeded', 0
isr_msg6: db '#UD Invalid Opcode', 0
isr_msg7: db '#NM Device Not Available', 0
isr_msg8: db '#DF Double Fault', 0
isr_msg9: db 'Coprocessor Segment Overrun', 0
isr_msg10: db '#TS Invalid TSS', 0
isr_msg11: db '#NP Segment Not Present', 0
isr_msg12: db '#SS Stack-Segment Fault', 0
isr_msg13: db '#GP General Protection', 0
isr_msg14: db '#PF Page Fault', 0
isr_msg15: db '#15 Intel reserved', 0
isr_msg16: db '#MF x87 FPU Floating-Point Error', 0
isr_msg17: db '#AC Alignment Check', 0
isr_msg18: db '#MC Machine Check', 0
isr_msg19: db '#XM SIMD Floating-Point Exception', 0
isr_msg20: db '#VE Virtualization Exception', 0

;;
;; Rutinas de atención de las EXCEPCIONES
;;

; TODO: Se pueden mejorar los handlers de excepciones. 
; Tendrían que guardar información de la excepción 
; (como el código de error) en app_data para poder 
; darsela al desarrollador o usuario de la aplicación.

%macro ISR_HANDLER 1
    global _isr%1
    _isr%1:
        mov rcx, isr_msg%1
        jmp generic_isr
%endmacro

ISR_HANDLER 0
ISR_HANDLER 1
ISR_HANDLER 2
ISR_HANDLER 3
ISR_HANDLER 4
ISR_HANDLER 5
ISR_HANDLER 6
ISR_HANDLER 7
ISR_HANDLER 8
ISR_HANDLER 9
ISR_HANDLER 10
ISR_HANDLER 11
ISR_HANDLER 12
ISR_HANDLER 13
ISR_HANDLER 14
ISR_HANDLER 15
ISR_HANDLER 16
ISR_HANDLER 17
ISR_HANDLER 18
ISR_HANDLER 19
ISR_HANDLER 20

isr_file_name: db __FILE__, 0
isr_function_name: db 'generic_isr', 0

generic_isr:
    cmp qword [app_data + APP_DATA_RUNNING_STATUS], 0
    je .panic
    ; Hubo un panic durante una aplicación.
    ; Vamos a intentar restaurar el contexto del kernel.
    mov rdi, -1
    jmp exit

    .panic:
    mov rax, 0
    mov rdi, isr_function_name
    mov rsi, __LINE__
    mov rdx, isr_file_name
    jmp kernel_panic


; ISRs de 21 a 31 reservados por intel

; Sirve para evitar interrupciones espurias
; La manera de manejarlas es salir sin señalizar al local apic que fue atendida
global _isr_spurious
_isr_spurious:
    xchg bx, bx
    iretq

;; TODO: Activar el pit
;;
;; Rutina de atención del RELOJ
;; ----------------------------
global _isr32
_isr32:
    pushaq
        call timer_tick
    popaq
    jmp end_of_interrupt

;;
;; Rutina de atención del TECLADO
;; ------------------------------

global _isr33
_isr33:
    pushaq
	    ; Obtenemos el scan code
        in al, 0x60
        mov dil, al
        call handle_key_event
    popaq
    jmp end_of_interrupt

;;
;; Rutina de atención del Real Time Clock
;; --------------------------------------

global _isr40
_isr40:
    pushaq
        call timer_tick
        ; Necesitamos leer el estado del reloj también, si no, no vuelve
        mov rdi, 0x0C
        call cmos_readb
    popaq
    jmp end_of_interrupt
;;
;; Rutina de atención de los discos ATA
;; ------------------------------------

;; Estas rutinas son temporales y se ocupan de atrapar interrupciones espurias
;; o no deseadas.

;; Primary ATA Hard Disk
global _isr46
_isr46:
    pushaq
        xchg bx, bx
    popaq
    jmp end_of_interrupt

;; Secondary ATA Hard Disk
global _isr47
_isr47:
    pushaq
        xchg bx, bx
    popaq
    jmp end_of_interrupt

;; Exit event
global _isr48
_isr48:
    ; Modifico la dirección de retorno
    jump_if_not_bsp .no_bsp
    ; ======== BSP ========
    ; Salta a execute.finished para volver a la consola.
    mov qword [rsp], execute_end
    jmp end_of_interrupt
    ; ======== APS ========
    .no_bsp:
    ; Salta a reset_stack_awe para prepararse para la próxima aplicación.
    mov qword [rsp], reset_stack_awe

;; WARNING: ESTA PEGADO A END_OF_INTERRUPT A PROPOSITO ;;

; En el local_apic_address habitualmente va a estar 0xFEE00000
%define eoi_register_apic_offset 0xB0

; Toda interrupción que viene por ioapic debería saltar acá
end_of_interrupt:
    push rax
    mov rax, [local_apic_address]
    mov DWORD [rax + eoi_register_apic_offset], 0
    pop rax
    iretq
