#include <defines.h>
#include <hdd.h>
#include <idt.h>
#include <io.h>
#include <kernel.h>
#include <keyboard.h>
#include <kmalloc.h>
#include <multicore_boot.h>
#include <pci.h>
#include <print.h>
#include <screen.h>
#include <shell.h>
#include <syscall.h>

static const char msg_done[] = "Done!\n";

noreturn void kernel_main() {
    // Limpiamos la pantalla
    screen_clear();

    print("[BSP] Initializing IDT... ");
    idt_init();
    print(msg_done);

    // Aca se inicializa tambien el IOAPIC y se activan interrupciones
    print("[BSP] Initializing Multicore... ");
    multiprocessor_init();
    print(msg_done);

    // TODO: Soportar relocations y sacar esto
    initialize_syscall_table((void*)SYSCALL_TABLE);

    print("[BSP] Enumerating PCI Buses...\n");
    pci_check_buses();

    print("[BSP] Initializing Kernel Heap... ");
    kheap_init();
    print(msg_done);

    print("[BSP] Initializing IO... ");
    io_init();
    print(msg_done);

    print("[BSP] Initializing Shell...\n");
    shell_init();
}
