#include <core_id.h>
#include <defines.h>
#include <i386.h>
#include <io.h>
#include <lapic.h>
#include <loader.h>
#include <syscall.h>
#include <timer.h>
#include <utils.h>

uintptr_t syscall_table[] = {
    (uintptr_t)&exit,      (uintptr_t)&core_id,  (uintptr_t)&core_count,
    (uintptr_t)&sleep,     (uintptr_t)&create,   (uintptr_t)&open,
    (uintptr_t)&close,     (uintptr_t)&read,     (uintptr_t)&write,
    (uintptr_t)&seek,      (uintptr_t)&tell,     (uintptr_t)&type,
    (uintptr_t)&opendir,   (uintptr_t)&closedir, (uintptr_t)&readdir,
    (uintptr_t)&rewinddir,
};

void initialize_syscall_table(void* address) {
    memcpy(address, syscall_table, sizeof(syscall_table));
}

extern application_data app_data;

void exit(int64_t exit_code) {
    uint64_t was_running =
        __atomic_exchange_n(&app_data.running_status, 0, __ATOMIC_ACQ_REL);
    // Si todavía nadie inició la terminación...
    if (was_running) {
        // Guardamos código de terminación y procesador.
        app_data.exit_code = exit_code;
        app_data.exit_core_id = core_id();
        intr_command_register ipi_command;
        // TODO: reemplazar valores hardcodeados por defines. Mandamos una
        // interrupción a todos incluyéndonos vectorizada en 48.
        // El destino es ignorado porque vamos a utilizar un shorthand.
        lapic_init_ipi(&ipi_command, FIXED, 48, 0, ALL_INCLUDING_SELF);
        lapic_send_ipi(&ipi_command);
    }
    // Activamos las interrupciones por las dudas.
    // Usamos sti() y no sti_safe() porque vamos a pisar los flags al volver.
    sti();
    while (1)
        hlt();
}
