#include <ext2_test.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ext2.h>
#include <block_cache.h>
#include <io_clash.h>

extern fs_info fs;

FILE* fp;

int main(int argc, char *argv[]) {
    if (!(fp = fopen("hdd.img", "rb+"))) {
        printf("hdd.img not found\n");
        return 1;
    }
    io_init();
    print_fs_data();
    write_test();
    dir_test();
    error_test();
    truncate_test();
    io_kill();
    fclose(fp);
    return 0;
}

void truncate_test() {
    assert(!sys_create("truncate_prueba0.txt", 0x8));
    int64_t fd = sys_open("truncate_prueba0.txt");
    assert(fd > 0);
    for (int i = 0; i <= 1024; i++) {
        assert(!sys_truncate(fd, i));
        assert(!sys_seek(fd, 0, SEEK_END));
        assert(sys_tell(fd) == i);
    }
    for (int i = 1024; i >= 0; i--) {
        assert(!sys_truncate(fd, i));
        assert(!sys_seek(fd, 0, SEEK_END));
        assert(sys_tell(fd) == i);
    }
    sys_close(fd);
}


void write_test() {

    char alphabet[] = "abcdefghijklmnopqrstuvwxyz"; // 26 chars + '\0'
    unsigned int len = 26;
    size_t times = 200000;


    // testeo create
    assert(!sys_create("write_prueba0.txt", 0x8));
    assert(!sys_create("write_prueba1.txt", 0x8));
    assert(!sys_create("write_prueba2.txt", 0x8));
    assert(!sys_create("write_prueba3.txt", 0x8));
    assert(!sys_create("write_prueba4.txt", 0x8));
    assert(!sys_create("write_prueba5.txt", 0x8));
    assert(!sys_create("write_prueba6.txt", 0x8));
    assert(!sys_create("write_prueba7.txt", 0x8));
    assert(!sys_create("write_prueba8.txt", 0x8));
    assert(!sys_create("write_prueba9.txt", 0x8));
    assert(!sys_create("write_pruebaa.txt", 0x8));
    assert(!sys_create("write_pruebab.txt", 0x8));
    assert(!sys_create("write_pruebac.txt", 0x8));
    assert(!sys_create("write_pruebad.txt", 0x8));
    assert(!sys_create("write_pruebae.txt", 0x8));
    assert(!sys_create("write_pruebaf.txt", 0x8));
    assert(!sys_create("write_pruebag.txt", 0x8));
    assert(!sys_create("write_pruebah.txt", 0x8));
    assert(!sys_create("write_pruebai.txt", 0x8));
    assert(!sys_create("write_pruebaj.txt", 0x8));
    assert(!sys_create("write_pruebak.txt", 0x8));
    assert(!sys_create("write_pruebal.txt", 0x8));
    assert(!sys_create("write_pruebam.txt", 0x8));
    assert(!sys_create("write_prueban.txt", 0x8));
    assert(!sys_create("write_pruebao.txt", 0x8));
    assert(!sys_create("write_pruebap.txt", 0x8));
    assert(!sys_create("write_pruebaq.txt", 0x8));
    assert(!sys_create("write_pruebar.txt", 0x8));
    assert(!sys_create("write_pruebas.txt", 0x8));
    assert(!sys_create("write_pruebat.txt", 0x8));
    assert(!sys_create("write_pruebau.txt", 0x8));
    assert(!sys_create("write_pruebav.txt", 0x8));
    assert(!sys_create("write_pruebaw.txt", 0x8));
    assert(!sys_create("write_pruebax.txt", 0x8));
    assert(!sys_create("write_pruebay.txt", 0x8));
    assert(!sys_create("write_pruebaz.txt", 0x8));



    // voy a testear write
    int f1 = sys_open("write_prueba1.txt");
    int f2 = sys_open("write_prueba2.txt");

    assert(f1 > 0);
    assert(f2 > 0);
     
    for(size_t i = 0; i < times; i++){
        assert(sys_write(f1, alphabet, len) == len);
        assert(sys_write(f2, alphabet, len) == len);
    }
    // cierro el segundo archivo
    sys_close(f2);
    
    // vuelvo al principio del primero
    sys_seek(f1, 0, 0);  

    const unsigned int buf_size = 99;
    char buf[buf_size];
    for(size_t i = 0; i<buf_size; i++)
        buf[i] = 0;

    // es importante que este numero sea coprimo con 26, asi podemos
    // recorrer todas las posiblidades
    for(size_t i = 0; i < times * len / sizeof(buf) - 1; i++){
        sys_read(f1, buf, buf_size);
        // chequeo que da lo que tiene que dar
        for(size_t j = 0; j < buf_size; j++){
            unsigned int idx = (buf_size * i + j) % len;
            
            assert(buf[j] == alphabet[idx]);
        }
    }
    sys_close(f1);

    // testeo remove (y truncate)
    assert(!sys_remove("write_prueba2.txt"));
    assert(!sys_remove("write_prueba4.txt"));
    assert(!sys_remove("write_prueba6.txt"));
    assert(!sys_remove("write_prueba8.txt"));
    assert(!sys_remove("write_pruebaa.txt"));
    assert(!sys_remove("write_pruebac.txt"));
    assert(!sys_remove("write_pruebae.txt"));
    assert(!sys_remove("write_pruebag.txt"));
}


void dir_test(){
    // testeo create
    assert(!sys_create("write_dir0", 0x4));
    assert(!sys_create("write_dir1", 0x4));
    assert(!sys_create("write_dir2", 0x4));
    assert(!sys_create("write_dir3", 0x4));
    assert(!sys_create("write_dir4", 0x4));
    assert(!sys_create("write_dir5", 0x4));
    assert(!sys_create("write_dir6", 0x4));
    assert(!sys_create("write_dir7", 0x4));
    assert(!sys_create("write_dir8", 0x4));
    assert(!sys_create("write_dir9", 0x4));


    assert(!sys_create("write_dir0/test_file0.txt", 0x8));
    assert(!sys_create("write_dir0/test_file1.txt", 0x8));
    assert(!sys_create("write_dir0/test_file2.txt", 0x8));
    assert(!sys_create("write_dir0/test_file3.txt", 0x8));
    assert(!sys_create("write_dir0/test_file4.txt", 0x8));


    assert(!sys_create("write_dir0/test_dir0", 0x4));
    assert(!sys_create("write_dir0/test_dir1", 0x4));
    assert(!sys_create("write_dir0/test_dir2", 0x4));
    assert(!sys_create("write_dir0/test_dir3", 0x4));
    assert(!sys_create("write_dir0/test_dir4", 0x4));


    assert(!sys_remove("write_dir1"));
    assert(!sys_remove("write_dir0/test_file2.txt"));
    assert(!sys_remove("write_dir0/test_dir3"));   
    
    assert(sys_remove("write_dir0/test_dir3"));   
}


void error_test(){
    assert(sys_remove("asdfasdfasdf"));   

    assert(!sys_create("error_test.txt", 0x8));
    assert(sys_create("error_test.txt", 0x8));

    int f1 = sys_open("error_test.txt");
    int f2 = sys_open("this_produces_an_error");

    assert(f1 > 0);
    assert(f2 < 0);

    assert(!sys_close(f1));
    assert(sys_close(f2));
}




void print_fs_data() {
    ext2_superblock* sb = fs.sb; 
    printf("Block Count: %i \n", sb->number_blocks);
    printf("Inode Count: %i \n", sb->number_inodes);
    printf("Group Count: %li \n", fs.group_count);
    printf("Blocks per Group: %i \n", sb->blocks_per_group);
    printf("Inodes per group: %i \n", sb->inodes_per_group);
    printf("Free blocks: %i \n", sb->free_blocks);
    printf("Free inodes: %i \n", sb->free_inodes);
    printf("Block size: %li \n", fs.block_size);
    printf("Inode size: %i \n", sb->inode_size);
    // printf("Block bitmap del primer grupo: %i \n", group_table[0].bn_block_bitmap);
    // printf("Block bitmap del segundo grupo: %i \n", group_table[1].bn_block_bitmap);
    int file = sys_open("/docs/prueba1.txt");
    char input[51] = {0};
    sys_read(file, input, 50);
    printf(input);
    sys_close(file);
    int file2 = sys_open("/docs/docs/prueba2.txt");
    char input2[51] = {0};
    sys_read(file2, input2, 50);
    printf(input2);
    sys_close(file2);
    int file3 = sys_open("/docs/docs/prueba3.txt");
    char input3[51] = {0};
    sys_read(file3, input3, 50);
    printf(input3);
    sys_close(file3);
    extract_file("/docs/prueba1.txt", "./dump/test1.txt");
    sys_opendir("asd");
    printf("Block Cache Final Count: %li \n", fs.cache->count);
}

void extract_file(char* source_path, char* dest_path) {
    char buf[4096];
    int64_t nread;
    int64_t fd_from = sys_open(source_path);
    if (fd_from < 0) 
        return;
    FILE* fd_to = fopen(dest_path, "w");
    while ((nread = sys_read(fd_from, buf, sizeof(buf)))) {
        if (nread < 0)
            return;
        char *out_ptr = buf;
        uint64_t nwritten;
        do {
            nwritten = fwrite(out_ptr, 1, nread, fd_to);
            if (nwritten >= 0) {
                nread -= (int64_t) nwritten;
                out_ptr += nwritten;
            }
        } while (nread > 0);
    }
    sys_close(fd_from);
    fclose(fd_to);
}

void* kmalloc(uint64_t size) {
    return malloc(size);    
}

void kfree(void* data) {
    free(data);    
}

void hdd_read(uint32_t lba_address, uint32_t sectors, void* buffer) {
    fseek(fp, lba_address * ATA_SECTSIZE, SEEK_SET);
    fread(buffer, ATA_SECTSIZE, sectors, fp);
}

void hdd_write(uint32_t lba_address, uint32_t sectors, void* buffer) {
    fseek(fp, lba_address * ATA_SECTSIZE, SEEK_SET);
    fwrite(buffer, ATA_SECTSIZE, sectors, fp);
} 

uint64_t screen_write(void* buffer, uint64_t count) {
    return fwrite((char *) buffer, 1, count, stdout);
}

uint64_t screen_read(void* buffer, uint64_t count) {
    return 0;
}

uint64_t screen_write_err(void *buffer, uint64_t count) {
    return fwrite((char *) buffer, 1, count, stderr);
}

void kernel_panic(const char* function,
                  const uint64_t line,
                  const char* file,
                  const char* message, ...) {
    va_list list;
    fprintf(stderr, "[KERNEL_PANIC]: ");
    va_start(list, message);
    vfprintf(stderr, message, list);
    va_end(list);
    fprintf(stderr, "\nFuncion que produjo el error: %s\n", function);
    fprintf(stderr, "(Archivo:Linea) %s:%lu\n", file, line);
    _Exit(1); 
}
